#!/usr/bin/env python

""" Colorize parts of a mup generated ps file.
    
    Please read the file README for more details.

    Caution: we're assuming that the mup output is very consistent!
    Caution: there's little to no error trapping done!

    Bob van der Poel, September 2011.  <bob@mellowood.ca>

    VERSION 1.0
"""

import sys, os

stdinout=None

# IMPORTANT  IMPORTANT  IMPORTANT
# if you have an rc file installed making changes
# to this section will have NO EFFECT. (Don't ask how
# I figured this out!!!

# The colors
colorRepeat = "0.13 0.55 0.13"   # repeats: forest green
colorCoda   = "1.0 0.0 0.2"      # coda, sign: red
colorKey    = "0.0 0.0 0.8"      # key change: dark blue 
colorTime   = "0.0 0.0 0.8"      # time change: dark blue

# Options
showRepeat  = 1
showKeysig  = 1
showKeysig2 = 0
showDC      = 1
showCoda    = 1
showSign    = 1
showTime    = 1

# End IMPORTANT

new = []         # list of the new PS file
lastSharp = 0    # flag for tracking sharps in keysigs
lastFlat  = 0    # flag for tracking flats in keysigs
lastNat   = 0

def addcolor(ps, color, comment):
    """ Return a PS string with the color inserted. """

    return "gsave %s setrgbcolor %s grestore %% %s" % (color, ps, comment)


#############################
# Init section. Command line args, etc.

l = len(sys.argv[1:])
if l == 0:
    stdinout=1
elif l == 1:
    infile = outfile = sys.argv[1]
elif l == 2:
    infile, outfile = sys.argv[1:3]
else:
    print  "mup-colorize: Usage [infile] [outfile]"
    print  "Colors MUP generated PS for \"sign\" and \"repeat\"."
    sys.exit(1)

if stdinout:
    psfile = sys.stdin
else:
    try:
        psfile = file(infile, 'r')
    except:
        print "ERROR: Can't open '%s' for reading." % infile
        sys.exit(1)

# Read the RC file
rcFile = os.path.expanduser("~/.mup-colorizerc")
if os.path.exists(rcFile):
    try:
        execfile( rcFile )
    except:
        print "Error reading %s:  %s" % (rcFile, sys.exc_info()[0] )

############################################################
# Start of main loop. Read file line by line and process

while 1:
    a=psfile.readline()

    if not a:    # break on EOF
        break

    a=a.rstrip()  # strip off trailing blanks and CR

    # "sign" This is three coords and "sign"

    if a.endswith("sign") and len(a.split())==4:
        if showSign:
            a = addcolor(a, colorCoda, 'sign')
        new.append(a)

    elif a.endswith("coda") and len(a.split())==4:
        if showCoda:
            a = addcolor(a, colorCoda, 'coda')
        new.append(a)

    elif a.endswith("show") and a.startswith("(") and \
            ("D.S." in a or "D.C." in a \
              or "D. S." in a or "D. C." in a \
              or ("Coda" in a)):
        if showDC:
            a = addcolor(a, colorCoda, 'D.S., D.C. Coda txt')
        new.append(a)

    # Repeats follow S_BAR/S_CELFSIG. Not all are repeats, so check
    # for a line ending in "dot". We need at least 2 dot lines.
    # note that the RED code is inserted before the first dot, that
    # way the clef/key sig is not set.

    elif a.startswith("%") and (a.endswith("S_BAR") or a.endswith("S_CLEFSIG")):
        tmp=[a]
        while 1:
            a=psfile.readline()
            if not a:   # unexpected EOF, should be error trapped?
                break
            a=a.rstrip()
            if a.startswith('%'):  # end of repeat sign section
                tmp.append(a)
                break
            tmp.append(a)

        if showRepeat:
            sFlag=[]
            eFlag=None
            for i,l in enumerate(tmp):
                if l.endswith("dot"):
                    sFlag.append(i)
            if len(sFlag)>1:
                for i,l in enumerate(tmp):
                    if i>sFlag[0] and (l.startswith("%") or l.endswith("findfont")
                                       or l=='newpath' or l=='gsave'):
                        eFlag=i
                        break

            if len(sFlag)>1 and eFlag:
                tmp.insert(sFlag[0], "gsave %s setrgbcolor %% start repeat colorization"
                           % colorRepeat)
                tmp.insert(eFlag, 'grestore  % end repeat colorization')

        # Now parse the stuff in the bar start again and colorize flat/sharp/nat
        # get accidental counts
        if showKeysig:
            flats = 0
            nats = 0
            sharps = 0
            for l in tmp:
                if   l.endswith('nat'):   nats += 1
                elif l.endswith('sharp'): sharps+=1
                elif l.endswith('flat'):  flats+=1
            # if changed from last parse, colorize each accidental line
            if nats != lastNat or sharps != lastSharp or flats != lastFlat:
                for i,l in enumerate(tmp):
                    if l.endswith('nat') or l.endswith('sharp') or l.endswith('flat'):
                        tmp[i] = addcolor(l, colorKey, 'nat, # or b')

            # save accidental b/# count for next time. Note: We don't save 'n' so that
            # only the actual key change is highlighted.
            if nats or sharps or flats:
                lastSharp = sharps
                lastFlat  = flats
                if showKeysig2:
                    lastNat   = nats
                else:
                    lastNat = 0

        if showTime:
            # Parse for timesig 'common' or 'cut'.
            for i,l in enumerate(tmp):
                if (l.endswith('cut') or l.endswith('com')) and len(l.split())==4:
                    tmp[i] = addcolor(l, colorTime, 'common/cut sig')


            # Parse for timesig '4/4', etc. We need to be careful not to get reh num 
            # so create a list of all (digit) show lines and discard the last found if
            # it's an odd number
            sigs=[]
            for i,l in enumerate(tmp):
                if (l.startswith('(') and l[1]in ("123456789") and len(l.split())==2):
                    sigs.append(i)
            if len(sigs) & 1:
                sigs = sigs[:-1]   # this should drop the reh num
            for i in sigs:
                tmp[i] = addcolor(tmp[i], colorTime, 'timesig')

        new.extend(tmp)

    # don't forget to save other lines
    else:
        new.append(a)

psfile.close()

if stdinout:
    psfile = sys.stdout
else:
    try:
        psfile = file(outfile, 'w')
    except:
        print "ERROR: can't open '%s' for write." % outfile
        sys.exit(1)

for a in new:
    psfile.write(a+"\n")
